@@ -45,8 +45,21 @@ opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
 class Enforcer(policy.Enforcer):
     """Responsible for loading and enforcing rules"""
 
-    def __init__(self):
+    def __init__(self, suppress_deprecation_warnings=False):
+        """Init an policy Enforcer.
+           :param suppress_deprecation_warnings: Whether to suppress the
+                                                 deprecation warnings.
+        """
         super(Enforcer, self).__init__(CONF, use_conf=True, overwrite=False)
+        # NOTE(gmann): Explictly disable the warnings for policies
+        # changing their default check_str. For new RBAC, all the policy
+        # defaults have been changed and warning for each policy started
+        # filling the logs limit for various tool.
+        # Once we move to new defaults only world then we can enable these
+        # warning again.
+        self.suppress_default_change_warnings = True
+        if suppress_deprecation_warnings:
+            self.suppress_deprecation_warnings = True
         self.register_defaults(policies.list_rules())
         if CONF.enforce_secure_rbac and CONF.oslo_policy.enforce_new_defaults:
             LOG.warning(_LW(
@@ -186,13 +199,11 @@ class ImageRepoProxy(glance.domain.proxy.Repo):
 
     def save(self, image, from_state=None):
         target = dict(image.target)
-        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'modify_image', target)
         return super(ImageRepoProxy, self).save(image, from_state=from_state)
 
     def add(self, image):
         target = dict(image.target)
-        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'add_image', target)
         return super(ImageRepoProxy, self).add(image)
 
@@ -220,7 +231,6 @@ class ImageProxy(glance.domain.proxy.Image):
     @visibility.setter
     def visibility(self, value):
         target = dict(self.target)
-        target['project_id'] = target.get('owner', None)
         _enforce_image_visibility(self.policy, self.context, value, target)
         self.image.visibility = value
 
@@ -243,14 +253,12 @@ class ImageProxy(glance.domain.proxy.Image):
 
     def delete(self):
         target = dict(self.target)
-        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'delete_image', target)
         return self.image.delete()
 
     def deactivate(self):
         LOG.debug('Attempting deactivate')
         target = dict(ImageTarget(self.image))
-        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'deactivate', target=target)
         LOG.debug('Deactivate allowed, continue')
         self.image.deactivate()
@@ -258,14 +266,12 @@ class ImageProxy(glance.domain.proxy.Image):
     def reactivate(self):
         LOG.debug('Attempting reactivate')
         target = dict(ImageTarget(self.image))
-        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'reactivate', target=target)
         LOG.debug('Reactivate allowed, continue')
         self.image.reactivate()
 
     def get_data(self, *args, **kwargs):
         target = dict(ImageTarget(self.image))
-        target['project_id'] = target.get('owner', None)
         self.policy.enforce(self.context, 'download_image', target)
         return self.image.get_data(*args, **kwargs)
 
@@ -551,12 +557,17 @@ class ImageTarget(abc.Mapping):
             yield key
         for key in getattr(self.target, 'extra_properties', {}).keys():
             yield key
+        for alias in ['project_id']:
+            yield alias
 
     def key_transforms(self, key):
-        if key == 'id':
-            key = 'image_id'
+        transforms = {
+            'id': 'image_id',
+            'project_id': 'owner',
+            'member_id': 'member',
+        }
 
-        return key
+        return transforms.get(key, key)
 
 
 # Metadef Namespace classes
