@@ -19,9 +19,11 @@ Routines for configuring Glance
 
 import logging
 import os
+import sys
 
 from oslo_config import cfg
 from oslo_middleware import cors
+from oslo_policy import opts
 from oslo_policy import policy
 from paste import deploy
 
@@ -159,16 +161,6 @@ Related Options:
 """)),
 ]
 
-_DEPRECATE_GLANCE_V1_MSG = _('The Images (Glance) version 1 API has been '
-                             'DEPRECATED in the Newton release and will be '
-                             'removed on or after Pike release, following '
-                             'the standard OpenStack deprecation policy. '
-                             'Hence, the configuration options specific to '
-                             'the Images (Glance) v1 API are hereby '
-                             'deprecated and subject to removal. Operators '
-                             'are advised to deploy the Images (Glance) v2 '
-                             'API.')
-
 common_opts = [
     cfg.BoolOpt('allow_additional_image_properties', default=True,
                 deprecated_for_removal=True,
@@ -278,59 +270,6 @@ Any negative value is interpreted as unlimited.
 Related options:
     * None
 
-""")),
-    # TODO(abashmak): Add choices parameter to this option:
-    # choices('glance.db.sqlalchemy.api',
-    #         'glance.db.registry.api',
-    #         'glance.db.simple.api')
-    # This will require a fix to the functional tests which
-    # set this option to a test version of the registry api module:
-    # (glance.tests.functional.v2.registry_data_api), in order to
-    # bypass keystone authentication for the Registry service.
-    # All such tests are contained in:
-    # glance/tests/functional/v2/test_images.py
-    cfg.StrOpt('data_api',
-               default='glance.db.sqlalchemy.api',
-                       deprecated_for_removal=True,
-                       deprecated_since="Queens",
-                       deprecated_reason=_("""
-Glance registry service is deprecated for removal.
-
-More information can be found from the spec:
-http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
-"""),
-               help=_("""
-Python module path of data access API.
-
-Specifies the path to the API to use for accessing the data model.
-This option determines how the image catalog data will be accessed.
-
-Possible values:
-    * glance.db.sqlalchemy.api
-    * glance.db.registry.api
-    * glance.db.simple.api
-
-If this option is set to ``glance.db.sqlalchemy.api`` then the image
-catalog data is stored in and read from the database via the
-SQLAlchemy Core and ORM APIs.
-
-Setting this option to ``glance.db.registry.api`` will force all
-database access requests to be routed through the Registry service.
-This avoids data access from the Glance API nodes for an added layer
-of security, scalability and manageability.
-
-NOTE: In v2 OpenStack Images API, the registry service is optional.
-In order to use the Registry API in v2, the option
-``enable_v2_registry`` must be set to ``True``.
-
-Finally, when this configuration option is set to
-``glance.db.simple.api``, image catalog data is stored in and read
-from an in-memory data structure. This is primarily used for testing.
-
-Related options:
-    * enable_v2_api
-    * enable_v2_registry
-
 """)),
     cfg.IntOpt('limit_param_default', default=25, min=1,
                help=_("""
@@ -411,7 +350,7 @@ Related options:
     * location_strategy
 
 """)),
-    # NOTE(flaper87): The policy.json file should be updated and the locaiton
+    # NOTE(flaper87): The policy.yaml file should be updated and the locaiton
     # related rules set to admin only once this option is finally removed.
     # NOTE(rosmaita): Unfortunately, this option is used to gate some code
     # paths; if the location related policies are set admin-only, then no
@@ -502,87 +441,29 @@ TeraBytes respectively. Note that there should not be any space between the
 value and unit. Value ``0`` signifies no quota enforcement. Negative values
 are invalid and result in errors.
 
+This has no effect if ``use_keystone_limits`` is enabled.
+
 Possible values:
     * A string that is a valid concatenation of a non-negative integer
       representing the storage value and an optional string literal
       representing storage units as mentioned above.
 
 Related options:
-    * None
+    * use_keystone_limits
 
 """)),
-    cfg.BoolOpt('enable_v2_api',
-                default=True,
-                deprecated_reason=_('The Images (Glance) version 1 API has '
-                                    'been DEPRECATED in the Newton release. '
-                                    'It will be removed on or after Pike '
-                                    'release, following the standard '
-                                    'OpenStack deprecation policy. Once we '
-                                    'remove the Images (Glance) v1 API, only '
-                                    'the Images (Glance) v2 API can be '
-                                    'deployed and will be enabled by default '
-                                    'making this option redundant.'),
-                deprecated_since='Newton',
+    cfg.BoolOpt('use_keystone_limits', default=False,
                 help=_("""
-Deploy the v2 OpenStack Images API.
+Utilize per-tenant resource limits registered in Keystone.
 
-When this option is set to ``True``, Glance service will respond
-to requests on registered endpoints conforming to the v2 OpenStack
-Images API.
-
-NOTES:
-    * If this option is disabled, then the ``enable_v2_registry``
-      option, which is enabled by default, is also recommended
-      to be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v2_registry
-
-""")),
-    cfg.BoolOpt('enable_v1_registry',
-                default=True,
-                deprecated_reason=_DEPRECATE_GLANCE_V1_MSG,
-                deprecated_since='Newton',
-                help=_("""
-                    DEPRECATED FOR REMOVAL
-""")),
-    cfg.BoolOpt('enable_v2_registry',
-                default=True,
-                deprecated_for_removal=True,
-                deprecated_since="Queens",
-                deprecated_reason=_("""
-Glance registry service is deprecated for removal.
-
-More information can be found from the spec:
-http://specs.openstack.org/openstack/glance-specs/specs/queens/approved/glance/deprecate-registry.html
-"""),
-                help=_("""
-Deploy the v2 API Registry service.
-
-When this option is set to ``True``, the Registry service
-will be enabled in Glance for v2 API requests.
-
-NOTES:
-    * Use of Registry is optional in v2 API, so this option
-      must only be enabled if both ``enable_v2_api`` is set to
-      ``True`` and the ``data_api`` option is set to
-      ``glance.db.registry.api``.
-
-    * If deploying only the v1 OpenStack Images API, this option,
-      which is enabled by default, should be disabled.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * enable_v2_api
-    * data_api
+Enabling this feature will cause Glance to retrieve limits set in keystone
+for resource consumption and enforce them against API users. Before turning
+this on, the limits need to be registered in Keystone or all quotas will be
+considered to be zero, and thus reject all new resource requests.
 
+These per-tenant resource limits are independent from the static
+global ones configured in this config file. If this is enabled, the
+relevant static global limits will be ignored.
 """)),
     cfg.HostAddressOpt('pydev_worker_debug_host',
                        sample_default='localhost',
@@ -700,13 +581,82 @@ Related options:
 
     Related options:
         * [DEFAULT]/node_staging_uri""")),
+    cfg.BoolOpt('enforce_secure_rbac', default=False,
+                deprecated_for_removal=True,
+                deprecated_reason=_("""
+This option has been introduced to require operators to opt into enforcing
+authorization based on common RBAC personas, which is EXPERIMENTAL as of the
+Wallaby release. This behavior will be the default and STABLE in a future
+release, allowing this option to be removed.
+"""),
+                deprecated_since='Wallaby',
+                help=_("""
+Enforce API access based on common persona definitions used across OpenStack.
+Enabling this option formalizes project-specific read/write operations, like
+creating private images or updating the status of shared image, behind the
+`member` role. It also formalizes a read-only variant useful for
+project-specific API operations, like listing private images in a project,
+behind the `reader` role.
+
+Operators should take an opportunity to understand glance's new image policies,
+audit assignments in their deployment, and update permissions using the default
+roles in keystone (e.g., `admin`, `member`, and `reader`).
+
+Related options:
+    * [oslo_policy]/enforce_new_defaults
+""")),
+    cfg.StrOpt('worker_self_reference_url',
+               default=None,
+               help=_("""
+The URL to this worker.
+
+If this is set, other glance workers will know how to contact this one
+directly if needed. For image import, a single worker stages the image
+and other workers need to be able to proxy the import request to the
+right one.
+
+If unset, this will be considered to be `public_endpoint`, which
+normally would be set to the same value on all workers, effectively
+disabling the proxying behavior.
+
+Possible values:
+    * A URL by which this worker is reachable from other workers
+
+Related options:
+    * public_endpoint
+
+""")),
 ]
 
+wsgi_opts = [
+    cfg.IntOpt('task_pool_threads',
+               default=16,
+               min=1,
+               help=_("""
+The number of threads (per worker process) in the pool for processing
+asynchronous tasks. This controls how many asynchronous tasks (i.e. for
+image interoperable import) each worker can run at a time. If this is
+too large, you *may* have increased memory footprint per worker and/or you
+may overwhelm other system resources such as disk or outbound network
+bandwidth. If this is too small, image import requests will have to wait
+until a thread becomes available to begin processing.""")),
+    cfg.StrOpt('python_interpreter',
+               default=sys.executable,
+               help=_("""
+Path to the python interpreter to use when spawning external
+processes. By default this is sys.executable, which should be the
+same interpreter running Glance itself. However, in some situations
+(i.e. uwsgi) this may not actually point to a python interpreter
+itself.""")),
+]
+
+
 CONF = cfg.CONF
 CONF.register_opts(paste_deploy_opts, group='paste_deploy')
 CONF.register_opts(image_format_opts, group='image_format')
 CONF.register_opts(task_opts, group='task')
 CONF.register_opts(common_opts)
+CONF.register_opts(wsgi_opts, group='wsgi')
 policy.Enforcer(CONF)
 
 
@@ -809,6 +759,12 @@ def set_config_defaults():
     """This method updates all configuration default values."""
     set_cors_middleware_defaults()
 
+    # TODO(gmann): Remove setting the default value of config policy_file
+    # once oslo_policy change the default value to 'policy.yaml'.
+    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+    DEFAULT_POLICY_FILE = 'policy.yaml'
+    opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
+
 
 def set_cors_middleware_defaults():
     """Update default configuration options for oslo.middleware."""
