@@ -33,7 +33,7 @@ LOG = logging.getLogger(__name__)
 
 DATA = {
     'images': {},
-    'members': {},
+    'members': [],
     'metadef_namespace_resource_types': [],
     'metadef_namespaces': [],
     'metadef_objects': [],
@@ -804,7 +804,7 @@ def image_update(context, image_id, image_values, purge_props=False,
     try:
         image = DATA['images'][image_id]
     except KeyError:
-        raise exception.ImageNotFound()
+        raise exception.ImageNotFound(image_id)
 
     location_data = image_values.pop('locations', None)
     if location_data is not None:
