@@ -50,8 +50,6 @@ $image_cache_dir/
   invalid/
   queue/
 """
-
-from __future__ import absolute_import
 from contextlib import contextmanager
 import errno
 import os
@@ -62,11 +60,12 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
+from oslo_utils import fileutils
 import six
 import xattr
 
 from glance.common import exception
-from glance.i18n import _, _LI, _LW
+from glance.i18n import _, _LI
 from glance.image_cache.drivers import base
 
 LOG = logging.getLogger(__name__)
@@ -115,8 +114,7 @@ class Driver(base.Driver):
                                                        reason=msg)
         else:
             # Cleanup after ourselves...
-            if os.path.exists(fake_image_filepath):
-                os.unlink(fake_image_filepath)
+            fileutils.delete_if_exists(fake_image_filepath)
 
     def get_cache_size(self):
         """
@@ -157,7 +155,6 @@ class Driver(base.Driver):
             entry['hits'] = self.get_hit_count(image_id)
 
             entries.append(entry)
-        entries.sort()  # Order by ID
         return entries
 
     def is_cached(self, image_id):
@@ -222,7 +219,7 @@ class Driver(base.Driver):
         """
         files = [f for f in get_all_regular_files(self.queue_dir)]
         for file in files:
-            os.unlink(file)
+            fileutils.delete_if_exists(file)
         return len(files)
 
     def delete_queued_image(self, image_id):
@@ -232,8 +229,7 @@ class Driver(base.Driver):
         :param image_id: Image ID
         """
         path = self.get_image_filepath(image_id, 'queue')
-        if os.path.exists(path):
-            os.unlink(path)
+        fileutils.delete_if_exists(path)
 
     def get_least_recently_accessed(self):
         """
@@ -280,7 +276,8 @@ class Driver(base.Driver):
             if self.is_queued(image_id):
                 LOG.debug("Removing image '%s' from queue after "
                           "caching it.", image_id)
-                os.unlink(self.get_image_filepath(image_id, 'queue'))
+                fileutils.delete_if_exists(
+                    self.get_image_filepath(image_id, 'queue'))
 
         def rollback(e):
             set_attr('error', encodeutils.exception_to_unicode(e))
@@ -432,12 +429,8 @@ def get_all_regular_files(basepath):
 
 
 def delete_cached_file(path):
-    if os.path.exists(path):
-        LOG.debug("Deleting image cache file '%s'", path)
-        os.unlink(path)
-    else:
-        LOG.warn(_LW("Cached image file '%s' doesn't exist, unable to"
-                     " delete") % path)
+    LOG.debug("Deleting image cache file '%s'", path)
+    fileutils.delete_if_exists(path)
 
 
 def _make_namespaced_xattr_key(key, namespace='user'):
