@@ -34,6 +34,7 @@ import socket
 import subprocess
 import sys
 import tempfile
+from testtools import content as ttc
 import textwrap
 import time
 from unittest import mock
@@ -69,6 +70,14 @@ else:
 CONF = cfg.CONF
 
 
+import glance.async_
+# NOTE(danms): Default to eventlet threading for tests
+try:
+    glance.async_.set_threadpool_model('eventlet')
+except RuntimeError:
+    pass
+
+
 @six.add_metaclass(abc.ABCMeta)
 class BaseServer(object):
     """
@@ -95,7 +104,6 @@ class BaseServer(object):
         self.show_image_direct_url = False
         self.show_multiple_locations = False
         self.property_protection_file = ''
-        self.enable_v2_api = True
         self.needs_database = False
         self.log_file = None
         self.sock = sock
@@ -103,7 +111,6 @@ class BaseServer(object):
         self.process_pid = None
         self.server_module = None
         self.stop_kill = False
-        self.use_user_token = True
         self.send_identity_credentials = False
 
     def write_conf(self, **kwargs):
@@ -275,7 +282,8 @@ class PosixServer(BaseServer):
         else:
             rc = test_utils.wait_for_fork(
                 self.process_pid,
-                expected_exitcode=expected_exitcode)
+                expected_exitcode=expected_exitcode,
+                force=False)
         # avoid an FD leak
         if self.sock:
             os.close(fd)
@@ -291,7 +299,8 @@ class PosixServer(BaseServer):
 
         if self.stop_kill:
             os.kill(self.process_pid, signal.SIGTERM)
-        rc = test_utils.wait_for_fork(self.process_pid, raise_error=False)
+        rc = test_utils.wait_for_fork(self.process_pid, raise_error=False,
+                                      force=self.stop_kill)
         return (rc, '', '')
 
 
@@ -389,7 +398,6 @@ class ApiServer(Server):
         self.log_file = os.path.join(self.test_dir, "api.log")
         self.image_size_cap = 1099511627776
         self.delayed_delete = delayed_delete
-        self.owner_is_tenant = True
         self.workers = 0
         self.scrub_time = 5
         self.image_cache_dir = os.path.join(self.test_dir,
@@ -404,12 +412,14 @@ class ApiServer(Server):
         self.image_location_quota = 2
         self.disable_path = None
 
+        secure_rbac = bool(os.getenv('OS_GLANCE_TEST_RBAC_DEFAULTS'))
+        self.enforce_secure_rbac = secure_rbac
+        self.enforce_new_defaults = secure_rbac
+
         self.needs_database = True
         default_sql_connection = SQLITE_CONN_TEMPLATE % self.test_dir
         self.sql_connection = os.environ.get('GLANCE_TEST_SQL_CONNECTION',
                                              default_sql_connection)
-        self.data_api = kwargs.get("data_api",
-                                   "glance.db.sqlalchemy.api")
         self.user_storage_quota = '0'
         self.lock_path = self.test_dir
 
@@ -423,27 +433,23 @@ class ApiServer(Server):
 
         self.conf_base = """[DEFAULT]
 debug = %(debug)s
-default_log_levels = eventlet.wsgi.server=DEBUG
+default_log_levels = eventlet.wsgi.server=DEBUG,stevedore.extension=INFO
 bind_host = %(bind_host)s
 bind_port = %(bind_port)s
 metadata_encryption_key = %(metadata_encryption_key)s
-use_user_token = %(use_user_token)s
 send_identity_credentials = %(send_identity_credentials)s
 log_file = %(log_file)s
 image_size_cap = %(image_size_cap)d
 delayed_delete = %(delayed_delete)s
-owner_is_tenant = %(owner_is_tenant)s
 workers = %(workers)s
 scrub_time = %(scrub_time)s
 send_identity_headers = %(send_identity_headers)s
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
-data_api = %(data_api)s
 sql_connection = %(sql_connection)s
 show_image_direct_url = %(show_image_direct_url)s
 show_multiple_locations = %(show_multiple_locations)s
 user_storage_quota = %(user_storage_quota)s
-enable_v2_api = %(enable_v2_api)s
 lock_path = %(lock_path)s
 property_protection_file = %(property_protection_file)s
 property_protection_rule_format = %(property_protection_rule_format)s
@@ -454,9 +460,11 @@ image_location_quota=%(image_location_quota)s
 location_strategy=%(location_strategy)s
 allow_additional_image_properties = True
 node_staging_uri=%(node_staging_uri)s
+enforce_secure_rbac=%(enforce_secure_rbac)s
 [oslo_policy]
 policy_file = %(policy_file)s
 policy_default_rule = %(policy_default_rule)s
+enforce_new_defaults=%(enforce_new_defaults)s
 [paste_deploy]
 flavor = %(deployment_flavor)s
 [store_type_location_strategy]
@@ -477,7 +485,7 @@ pipeline =
     rootapp
 
 [pipeline:glance-api-caching]
-pipeline = cors healthcheck versionnegotiation gzip unauthenticated-context
+pipeline = cors healthcheck versionnegotiation gzip context
  cache rootapp
 
 [pipeline:glance-api-cachemanagement]
@@ -500,15 +508,11 @@ pipeline = cors healthcheck versionnegotiation gzip context rootapp
 [composite:rootapp]
 paste.composite_factory = glance.api:root_app_factory
 /: apiversions
-/v1: apiv1app
 /v2: apiv2app
 
 [app:apiversions]
 paste.app_factory = glance.api.versions:create_resource
 
-[app:apiv1app]
-paste.app_factory = glance.api.v1.router:API.factory
-
 [app:apiv2app]
 paste.app_factory = glance.api.v2.router:API.factory
 
@@ -571,7 +575,6 @@ class ApiServerForMultipleBackend(Server):
         self.log_file = os.path.join(self.test_dir, "multiple_backend_api.log")
         self.image_size_cap = 1099511627776
         self.delayed_delete = delayed_delete
-        self.owner_is_tenant = True
         self.workers = 0
         self.scrub_time = 5
         self.image_cache_dir = os.path.join(self.test_dir,
@@ -590,8 +593,6 @@ class ApiServerForMultipleBackend(Server):
         default_sql_connection = SQLITE_CONN_TEMPLATE % self.test_dir
         self.sql_connection = os.environ.get('GLANCE_TEST_SQL_CONNECTION',
                                              default_sql_connection)
-        self.data_api = kwargs.get("data_api",
-                                   "glance.db.sqlalchemy.api")
         self.user_storage_quota = '0'
         self.lock_path = self.test_dir
 
@@ -602,27 +603,23 @@ class ApiServerForMultipleBackend(Server):
 
         self.conf_base = """[DEFAULT]
 debug = %(debug)s
-default_log_levels = eventlet.wsgi.server=DEBUG
+default_log_levels = eventlet.wsgi.server=DEBUG,stevedore.extension=INFO
 bind_host = %(bind_host)s
 bind_port = %(bind_port)s
 metadata_encryption_key = %(metadata_encryption_key)s
-use_user_token = %(use_user_token)s
 send_identity_credentials = %(send_identity_credentials)s
 log_file = %(log_file)s
 image_size_cap = %(image_size_cap)d
 delayed_delete = %(delayed_delete)s
-owner_is_tenant = %(owner_is_tenant)s
 workers = %(workers)s
 scrub_time = %(scrub_time)s
 send_identity_headers = %(send_identity_headers)s
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
-data_api = %(data_api)s
 sql_connection = %(sql_connection)s
 show_image_direct_url = %(show_image_direct_url)s
 show_multiple_locations = %(show_multiple_locations)s
 user_storage_quota = %(user_storage_quota)s
-enable_v2_api = %(enable_v2_api)s
 lock_path = %(lock_path)s
 property_protection_file = %(property_protection_file)s
 property_protection_rule_format = %(property_protection_rule_format)s
@@ -686,15 +683,11 @@ pipeline = cors healthcheck versionnegotiation gzip context rootapp
 [composite:rootapp]
 paste.composite_factory = glance.api:root_app_factory
 /: apiversions
-/v1: apiv1app
 /v2: apiv2app
 
 [app:apiversions]
 paste.app_factory = glance.api.versions:create_resource
 
-[app:apiv1app]
-paste.app_factory = glance.api.v1.router:API.factory
-
 [app:apiv2app]
 paste.app_factory = glance.api.v2.router:API.factory
 
@@ -759,7 +752,6 @@ class ScrubberDaemon(Server):
         self.policy_default_rule = 'default'
 
         self.send_identity_headers = False
-        self.admin_role = 'admin'
 
         self.conf_base = """[DEFAULT]
 debug = %(debug)s
@@ -818,7 +810,6 @@ class FunctionalTest(test_utils.BaseTestCase):
         conf_dir = os.path.join(self.test_dir, 'etc')
         utils.safe_mkdirs(conf_dir)
         self.copy_data_file('schema-image.json', conf_dir)
-        self.copy_data_file('policy.json', conf_dir)
         self.copy_data_file('property-protections.conf', conf_dir)
         self.copy_data_file('property-protections-policies.conf', conf_dir)
         self.property_file_roles = os.path.join(conf_dir,
@@ -826,7 +817,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         property_policies = 'property-protections-policies.conf'
         self.property_file_policies = os.path.join(conf_dir,
                                                    property_policies)
-        self.policy_file = os.path.join(conf_dir, 'policy.json')
+        self.policy_file = os.path.join(conf_dir, 'policy.yaml')
 
         self.api_server = ApiServer(self.test_dir,
                                     self.api_port,
@@ -852,6 +843,9 @@ class FunctionalTest(test_utils.BaseTestCase):
             self.addCleanup(self.cleanup)
             self._reset_database(self.api_server.sql_connection)
 
+    def _url(self, path):
+        return 'http://127.0.0.1:%d%s' % (self.api_port, path)
+
     def set_policy_rules(self, rules):
         fap = open(self.policy_file, 'w')
         fap.write(jsonutils.dumps(rules))
@@ -1167,7 +1161,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         conf_dir = os.path.join(self.test_dir, 'etc')
         utils.safe_mkdirs(conf_dir)
         self.copy_data_file('schema-image.json', conf_dir)
-        self.copy_data_file('policy.json', conf_dir)
         self.copy_data_file('property-protections.conf', conf_dir)
         self.copy_data_file('property-protections-policies.conf', conf_dir)
         self.property_file_roles = os.path.join(conf_dir,
@@ -1175,7 +1168,7 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         property_policies = 'property-protections-policies.conf'
         self.property_file_policies = os.path.join(conf_dir,
                                                    property_policies)
-        self.policy_file = os.path.join(conf_dir, 'policy.json')
+        self.policy_file = os.path.join(conf_dir, 'policy.yaml')
 
         self.api_server_multiple_backend = ApiServerForMultipleBackend(
             self.test_dir, self.api_port, self.policy_file, sock=api_sock)
@@ -1201,6 +1194,9 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
             self._reset_database(
                 self.api_server_multiple_backend.sql_connection)
 
+    def _url(self, path):
+        return 'http://127.0.0.1:%d%s' % (self.api_port, path)
+
     def set_policy_rules(self, rules):
         fap = open(self.policy_file, 'w')
         fap.write(jsonutils.dumps(rules))
@@ -1550,6 +1546,14 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             [filter:fakeauth]
             paste.filter_factory = glance.tests.utils:\
                 FakeAuthMiddleware.factory
+            [filter:cache]
+            paste.filter_factory = glance.api.middleware.cache:\
+            CacheFilter.factory
+            [filter:cachemanage]
+            paste.filter_factory = glance.api.middleware.cache_manage:\
+            CacheManageFilter.factory
+            [pipeline:glance-api-cachemanagement]
+            pipeline = context cache cachemanage rootapp
             [pipeline:glance-api]
             pipeline = context rootapp
             [composite:rootapp]
@@ -1584,6 +1588,8 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
                     group='store3')
         self.config(filesystem_store_datadir=self._store_dir('staging'),
                     group='os_glance_staging_store')
+        self.config(filesystem_store_datadir=self._store_dir('tasks'),
+                    group='os_glance_tasks_store')
 
         glance_store.create_multi_stores(CONF,
                                          reserved_stores=wsgi.RESERVED_STORES)
@@ -1596,7 +1602,7 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         self.setup_simple_paste()
         self.setup_stores()
 
-    def start_server(self):
+    def start_server(self, enable_cache=False):
         """Builds and "starts" the API server.
 
         Note that this doesn't actually "start" anything like
@@ -1604,8 +1610,17 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         to make it seem like the same sort of pattern.
         """
         config.set_config_defaults()
-        self.api = config.load_paste_app('glance-api',
+        root_app = 'glance-api'
+        if enable_cache:
+            root_app = 'glance-api-cachemanagement'
+            self.config(image_cache_dir=self._store_dir('cache'))
+
+        self.api = config.load_paste_app(root_app,
                                          conf_file=self.paste_config)
+        secure_rbac = bool(os.getenv('OS_GLANCE_TEST_RBAC_DEFAULTS'))
+        self.config(enforce_secure_rbac=secure_rbac)
+        self.config(enforce_new_defaults=secure_rbac,
+                    group='oslo_policy')
 
     def _headers(self, custom_headers=None):
         base_headers = {
@@ -1697,4 +1712,167 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         :param headers: Optional updates to the default set of headers
         :returns: A webob.Response object
         """
-        return self.api_request('DELETE', url, heaers=headers)
+        return self.api_request('DELETE', url, headers=headers)
+
+    def api_patch(self, url, *patches, headers=None):
+        """Perform a PATCH request against the API.
+
+        :param url: The *path* part of the URL to call (i.e. /v2/images)
+        :param patches: One or more patch dicts
+        :param headers: Optional updates to the default set of headers
+        :returns: A webob.Response object
+        """
+        if not headers:
+            headers = {}
+        headers['Content-Type'] = \
+            'application/openstack-images-v2.1-json-patch'
+        return self.api_request('PATCH', url, headers=headers,
+                                json=list(patches))
+
+    def _import_copy(self, image_id, stores, headers=None):
+        """Do an import of image_id to the given stores."""
+        body = {'method': {'name': 'copy-image'},
+                'stores': stores,
+                'all_stores': False}
+
+        return self.api_post(
+            '/v2/images/%s/import' % image_id,
+            headers=headers,
+            json=body)
+
+    def _import_direct(self, image_id, stores, headers=None):
+        """Do an import of image_id to the given stores."""
+        body = {'method': {'name': 'glance-direct'},
+                'stores': stores,
+                'all_stores': False}
+
+        return self.api_post(
+            '/v2/images/%s/import' % image_id,
+            headers=headers,
+            json=body)
+
+    def _import_web_download(self, image_id, stores, url, headers=None):
+        """Do an import of image_id to the given stores."""
+        body = {'method': {'name': 'web-download',
+                           'uri': url},
+                'stores': stores,
+                'all_stores': False}
+
+        return self.api_post(
+            '/v2/images/%s/import' % image_id,
+            headers=headers,
+            json=body)
+
+    def _create_and_upload(self, data_iter=None, expected_code=204,
+                           visibility=None):
+        data = {
+            'name': 'foo',
+            'container_format': 'bare',
+            'disk_format': 'raw'
+        }
+        if visibility:
+            data['visibility'] = visibility
+
+        resp = self.api_post('/v2/images',
+                             json=data)
+        self.assertEqual(201, resp.status_code, resp.text)
+        image = jsonutils.loads(resp.text)
+
+        if data_iter:
+            resp = self.api_put(
+                '/v2/images/%s/file' % image['id'],
+                headers={'Content-Type': 'application/octet-stream'},
+                body_file=data_iter)
+        else:
+            resp = self.api_put(
+                '/v2/images/%s/file' % image['id'],
+                headers={'Content-Type': 'application/octet-stream'},
+                data=b'IMAGEDATA')
+        self.assertEqual(expected_code, resp.status_code)
+
+        return image['id']
+
+    def _create_and_stage(self, data_iter=None, expected_code=204,
+                          visibility=None, extra={}):
+        data = {
+            'name': 'foo',
+            'container_format': 'bare',
+            'disk_format': 'raw',
+        }
+        if visibility:
+            data['visibility'] = visibility
+
+        data.update(extra)
+        resp = self.api_post('/v2/images',
+                             json=data)
+        image = jsonutils.loads(resp.text)
+
+        if data_iter:
+            resp = self.api_put(
+                '/v2/images/%s/stage' % image['id'],
+                headers={'Content-Type': 'application/octet-stream'},
+                body_file=data_iter)
+        else:
+            resp = self.api_put(
+                '/v2/images/%s/stage' % image['id'],
+                headers={'Content-Type': 'application/octet-stream'},
+                data=b'IMAGEDATA')
+        self.assertEqual(expected_code, resp.status_code)
+
+        return image['id']
+
+    def _wait_for_import(self, image_id, retries=10):
+        for i in range(0, retries):
+            image = self.api_get('/v2/images/%s' % image_id).json
+            if not image.get('os_glance_import_task'):
+                break
+            self.addDetail('Create-Import task id',
+                           ttc.text_content(image['os_glance_import_task']))
+            time.sleep(1)
+
+        self.assertIsNone(image.get('os_glance_import_task'),
+                          'Timed out waiting for task to complete')
+
+        return image
+
+    def _create_and_import(self, stores=[], data_iter=None, expected_code=202,
+                           visibility=None, extra={}):
+        """Create an image, stage data, and import into the given stores.
+
+        :returns: image_id
+        """
+        image_id = self._create_and_stage(data_iter=data_iter,
+                                          visibility=visibility,
+                                          extra=extra)
+
+        resp = self._import_direct(image_id, stores)
+        self.assertEqual(expected_code, resp.status_code)
+
+        if expected_code >= 400:
+            return image_id
+
+        # Make sure it becomes active
+        image = self._wait_for_import(image_id)
+        self.assertEqual('active', image['status'])
+
+        return image_id
+
+    def _get_latest_task(self, image_id):
+        tasks = self.api_get('/v2/images/%s/tasks' % image_id).json['tasks']
+        tasks = sorted(tasks, key=lambda t: t['updated_at'])
+        self.assertGreater(len(tasks), 0)
+        return tasks[-1]
+
+    def _create(self):
+        return self.api_post('/v2/images',
+                             json={'name': 'foo',
+                                   'container_format': 'bare',
+                                   'disk_format': 'raw'})
+
+    def _create_metadef_resource(self, path=None, data=None,
+                                 expected_code=201):
+        resp = self.api_post(path,
+                             json=data)
+        md_resource = jsonutils.loads(resp.text)
+        self.assertEqual(expected_code, resp.status_code)
+        return md_resource
