@@ -16,23 +16,33 @@
 from contextlib import contextmanager
 import datetime
 import os
+import tempfile
 import time
+from unittest import mock
 
 import fixtures
+import glance_store as store
+from oslo_config import cfg
 from oslo_utils import secretutils
 from oslo_utils import units
 import six
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
 
+from glance import async_
 from glance.common import exception
+from glance import context
+from glance import gateway as glance_gateway
 from glance import image_cache
+from glance.image_cache import prefetcher
+from glance.tests.unit import utils as unit_test_utils
 from glance.tests import utils as test_utils
 from glance.tests.utils import skip_if_disabled
 from glance.tests.utils import xattr_writes_supported
 
 FIXTURE_LENGTH = 1024
 FIXTURE_DATA = b'*' * FIXTURE_LENGTH
+CONF = cfg.CONF
 
 
 class ImageCacheTestCase(object):
@@ -510,6 +520,61 @@ class TestImageCacheSqlite(test_utils.BaseTestCase,
                     image_cache_max_size=5 * units.Ki)
         self.cache = image_cache.ImageCache()
 
+    @mock.patch('glance.db.get_api')
+    def _test_prefetcher(self, mock_get_db):
+        self.config(enabled_backends={'cheap': 'file'})
+        store.register_store_opts(CONF)
+        self.config(filesystem_store_datadir='/tmp', group='cheap')
+        store.create_multi_stores(CONF)
+
+        tempf = tempfile.NamedTemporaryFile()
+        tempf.write(b'foo')
+
+        db = unit_test_utils.FakeDB(initialize=False)
+        mock_get_db.return_value = db
+
+        ctx = context.RequestContext(is_admin=True, roles=['admin'])
+        gateway = glance_gateway.Gateway()
+        image_factory = gateway.get_image_factory(ctx,
+                                                  authorization_layer=False)
+        image_repo = gateway.get_repo(ctx, authorization_layer=False)
+        fetcher = prefetcher.Prefetcher()
+
+        # Create an image with no values set and queue it
+        image = image_factory.new_image()
+        image_repo.add(image)
+        fetcher.cache.queue_image(image.image_id)
+
+        # Image is not active, so it should fail to cache, but remain queued
+        self.assertFalse(fetcher.run())
+        self.assertFalse(fetcher.cache.is_cached(image.image_id))
+        self.assertTrue(fetcher.cache.is_queued(image.image_id))
+
+        # Set the disk/container format and give it a location
+        image.disk_format = 'raw'
+        image.container_format = 'bare'
+        image.status = 'active'
+        loc = {'url': 'file://%s' % tempf.name, 'metadata': {'store': 'cheap'}}
+        with mock.patch('glance.location._check_image_location'):
+            # FIXME(danms): Why do I have to do this?
+            image.locations = [loc]
+        image_repo.save(image)
+
+        # Image is now active and has a location, so it should cache
+        self.assertTrue(fetcher.run())
+        self.assertTrue(fetcher.cache.is_cached(image.image_id))
+        self.assertFalse(fetcher.cache.is_queued(image.image_id))
+
+    @mock.patch('glance.async_._THREADPOOL_MODEL', new=None)
+    def test_prefetcher_greenthread(self):
+        async_.set_threadpool_model('eventlet')
+        self._test_prefetcher()
+
+    @mock.patch('glance.async_._THREADPOOL_MODEL', new=None)
+    def test_prefetcher_native(self):
+        async_.set_threadpool_model('native')
+        self._test_prefetcher()
+
 
 class TestImageCacheNoDep(test_utils.BaseTestCase):
 
@@ -564,3 +629,20 @@ class TestImageCacheNoDep(test_utils.BaseTestCase):
 
         caching_iter = cache.get_caching_iter('dummy_id', None, iter(data))
         self.assertEqual(data, list(caching_iter))
+
+
+class TestImagePrefetcher(test_utils.BaseTestCase):
+    def setUp(self):
+        super(TestImagePrefetcher, self).setUp()
+        self.cache_dir = self.useFixture(fixtures.TempDir()).path
+        self.config(image_cache_dir=self.cache_dir,
+                    image_cache_driver='xattr',
+                    image_cache_max_size=5 * units.Ki)
+        self.prefetcher = prefetcher.Prefetcher()
+
+    def test_fetch_image_into_cache_without_auth(self):
+        with mock.patch.object(self.prefetcher.gateway,
+                               'get_repo') as mock_get:
+            self.prefetcher.fetch_image_into_cache('fake-image-id')
+            mock_get.assert_called_once_with(mock.ANY,
+                                             authorization_layer=False)
