@@ -12,8 +12,6 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
-from __future__ import absolute_import
 from unittest import mock
 
 import fixtures
@@ -260,11 +258,8 @@ class TestManage(TestManageBase):
         'glance.db.sqlalchemy.alembic_migrations.get_current_alembic_heads')
     @mock.patch(
         'glance.db.sqlalchemy.alembic_migrations.get_alembic_branch_head')
-    @mock.patch('glance.db.sqlalchemy.alembic_migrations.'
-                'place_database_under_alembic_control')
     @mock.patch('alembic.command.upgrade')
     def test_sync_db_is_already_sync(self, mock_upgrade,
-                                     mock_db_under_alembic_control,
                                      mock_get_alembic_branch_head,
                                      mock_get_current_alembic_heads):
         mock_get_current_alembic_heads.return_value = ['pike_contract01']
