@@ -249,8 +249,8 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         for output_log, event in zip(output_logs, expected_events):
             self.assertEqual('INFO', output_log['notification_type'])
             self.assertEqual(event['type'], output_log['event_type'])
-            self.assertDictContainsSubset(event['payload'],
-                                          output_log['payload'])
+            self.assertLessEqual(event['payload'].items(),
+                                 output_log['payload'].items())
         self.notifier.log = []
 
     def test_namespace_index(self):
@@ -570,7 +570,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                                    [{'namespace': NAMESPACE3}])
 
     def test_namespace_non_existing_delete_properties(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.namespace_controller.delete_properties,
                           request,
@@ -608,7 +608,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                                    [{'namespace': NAMESPACE3}])
 
     def test_namespace_non_existing_delete_objects(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.namespace_controller.delete_objects,
                           request,
@@ -645,7 +645,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                                    [{'namespace': NAMESPACE3}])
 
     def test_namespace_non_existing_delete_tags(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.namespace_controller.delete_tags,
                           request,
@@ -997,14 +997,14 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                           PROPERTY1)
 
     def test_property_delete_non_existing(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.property_controller.delete, request, NAMESPACE5,
                           PROPERTY2)
         self.assertNotificationsLog([])
 
     def test_property_delete_non_existing_namespace(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.property_controller.delete, request, NAMESPACE4,
                           PROPERTY1)
@@ -1114,7 +1114,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         property.type = 'string'
         property.title = 'title'
 
-        self.assertRaises(webob.exc.HTTPForbidden,
+        self.assertRaises(webob.exc.HTTPNotFound,
                           self.property_controller.create, request, NAMESPACE1,
                           property)
         self.assertNotificationsLog([])
@@ -1378,14 +1378,14 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
                           request, NAMESPACE3, OBJECT1)
 
     def test_object_delete_non_existing(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.object_controller.delete, request, NAMESPACE5,
                           OBJECT1)
         self.assertNotificationsLog([])
 
     def test_object_delete_non_existing_namespace(self):
-        request = unit_test_utils.get_fake_request()
+        request = unit_test_utils.get_fake_request(roles=['admin'])
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.object_controller.delete, request, NAMESPACE4,
                           OBJECT1)
@@ -1519,7 +1519,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         object.required = []
         object.properties = {}
 
-        self.assertRaises(webob.exc.HTTPForbidden,
+        self.assertRaises(webob.exc.HTTPNotFound,
                           self.object_controller.create, request, object,
                           NAMESPACE1)
         self.assertNotificationsLog([])
@@ -1827,7 +1827,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         rt = resource_types.ResourceTypeAssociation()
         rt.name = RESOURCE_TYPE2
         rt.prefix = 'pref'
-        self.assertRaises(webob.exc.HTTPForbidden, self.rt_controller.create,
+        self.assertRaises(webob.exc.HTTPNotFound, self.rt_controller.create,
                           request, rt, NAMESPACE1)
         self.assertNotificationsLog([])
 
@@ -2064,7 +2064,7 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
 
     def test_tag_create_non_visible_namespace(self):
         request = unit_test_utils.get_fake_request(tenant=TENANT2)
-        self.assertRaises(webob.exc.HTTPForbidden,
+        self.assertRaises(webob.exc.HTTPNotFound,
                           self.tag_controller.create, request,
                           NAMESPACE1, TAG1)
         self.assertNotificationsLog([])
@@ -2141,7 +2141,8 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         self.assertNotificationsLog([])
 
     def test_tag_update_non_existing(self):
-        request = unit_test_utils.get_fake_request(tenant=TENANT3)
+        request = unit_test_utils.get_fake_request(
+            tenant=TENANT3, roles=['admin'])
 
         tag = tags.MetadefTag()
         tag.name = TAG1
@@ -2152,7 +2153,8 @@ class TestMetadefsControllers(base.IsolatedUnitTest):
         self.assertNotificationsLog([])
 
     def test_tag_update_namespace_non_existing(self):
-        request = unit_test_utils.get_fake_request(tenant=TENANT3)
+        request = unit_test_utils.get_fake_request(
+            tenant=TENANT3, roles=['admin'])
 
         tag = tags.MetadefTag()
         tag.name = TAG1
